@@ -1,14 +1,14 @@
-# DMT
+# DMT_core
 # Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
 #
-# This file is part of DMT.
+# This file is part of DMT_core.
 #
-# DMT is free software: you can redistribute it and/or modify
+# DMT_core is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# DMT is distributed in the hope that it will be useful,
+# DMT_core is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -19,15 +19,18 @@ import os
 import re
 import shutil
 import _pickle as cpickle
+
+# import pickle as cpickle
 import copy
 import logging
 from DMT.core import DatabaseManager, read_data, DataFrame
 from DMT.config import DATA_CONFIG
 from pathlib import Path
+import pandas as pd
 
 
 class DutView(object):
-    """  DutView is the parent class of all DUTs in DMT.
+    """DutView is the parent class of all DUTs in DMT.
 
     All simulation back-end classes must inherit from this class and overwrite the methods that raise NotImplemented errors.
     This ensures that all classes in DMT work independent of the concrete simulation back-end.
@@ -69,6 +72,8 @@ class DutView(object):
         Number of parallel emitter fingers
     contact_config : str, optional
         String describing the physical contact config for this device. The possible value depends on the technology, example: 'CBEBC'.
+    inp_name   :  str
+        The name of the input file to be generated.
 
     Attributes
     -----------
@@ -79,8 +84,9 @@ class DutView(object):
     manager    :  :class:`~DMT.core.database_manager.DatabaseManager`
         The DatabaseManager object of the DutView, used to interact with its database.
     sim_folder :  str
+        Path to the simulation folder
     inp_name   :  str
-        The name of the input file to be generated. (TODO:Mario?)
+        The name of the input file to be generated.
     save_dir   : string
         This is the directory were the DutView object will save its object and database.
     dut_dir    : string
@@ -127,56 +133,79 @@ class DutView(object):
         Iterate through all keys in the DutView's database and try to clean the column names according to the internal DMT format.
     """
 
-    def __init__(self, database_dir, name, dut_type,
-    copy_va_files=False, force=False, separate_databases=False, list_copy=None, t_max=None, simulate_on_server=None,
-    simulator_command=None, simulator_arguments=None, technology=None, width=None, length=None, nfinger=None, contact_config=None,
-    ac_ports=None, reference_node=None, nodes=None, ports=None):
+    def __init__(
+        self,
+        database_dir,
+        name,
+        dut_type,
+        copy_va_files=False,
+        force=False,
+        separate_databases=False,
+        list_copy=None,
+        t_max=None,
+        simulate_on_server=None,
+        simulator_command=None,
+        simulator_arguments=None,
+        technology=None,
+        width=None,
+        length=None,
+        nfinger=None,
+        contact_config=None,
+        ac_ports=None,
+        reference_node=None,
+        nodes=None,
+        inp_name=None,
+    ):
         self._copy_va_files = copy_va_files
-        #files to be copied into simulation directory
+        # files to be copied into simulation directory
         if list_copy is None:
             self.list_copy = []
         else:
-            self.list_copy = list_copy
+            if not isinstance(list_copy, list):
+                self.list_copy = [list_copy]
+            else:
+                self.list_copy = list_copy
+
         self._list_va_file_contents = []
 
-        #attributes for data management
+        # attributes for data management
         self._separate_databases = separate_databases
-        self._data          =  {} # this is now hidden
+        self._data = {}  # this is now hidden
         if database_dir is None:
-            self._database_dir  =  DATA_CONFIG['directories']['database']
+            self._database_dir = DATA_CONFIG["directories"]["database"]
         else:
             if isinstance(database_dir, Path):
-                self._database_dir  =  database_dir
+                self._database_dir = database_dir.expanduser()
             else:
-                self._database_dir  =  Path(database_dir)
+                self._database_dir = Path(database_dir).expanduser()
 
-        self.dut_type       =  dut_type
+        self.dut_type = dut_type
 
         if os.path.isabs(name):
-            raise IOError("The DuT name must not be an valid absolute directory path!")
-        self.name           =  name
+            raise IOError("The DuT name must not be a valid absolute directory path!")
+        self.name = name
 
-        self.manager        =  DatabaseManager()
+        self.manager = DatabaseManager()
         if nodes is None:
-            self.nodes      =  dut_type.get_nodes()
+            self.nodes = dut_type.get_nodes()
         else:
             if isinstance(nodes, str):
-                self.nodes = nodes.split(',')
+                self.nodes = nodes.split(",")
             else:
                 self.nodes = nodes
 
         if ac_ports is None:
-            self.ac_ports       = self.nodes[0:2] # ports for AC simulations
+            self.ac_ports = self.nodes[0:2]  # ports for AC simulations
         else:
-            self.ac_ports       = ac_ports
+            self.ac_ports = ac_ports
 
-        self.reference_node  = reference_node
+        self.reference_node = reference_node
         if reference_node is None:
-            raise IOError('Reference node not specified. DMT does not support this.')
+            raise IOError("Reference node not specified. DMT does not support this.")
 
         # simulation attributes
-        self.inp_name       = None
-        self.sim_command    = simulator_command
+        self.inp_name = inp_name
+        self.sim_command = simulator_command
 
         if simulator_arguments is None:
             self.sim_args = []
@@ -189,67 +218,85 @@ class DutView(object):
                 self.del_db()
                 self.del_dut()
 
-        #try to reload this Dut if it already exists!
+        # try to reload this Dut if it already exists!
         if self.save_dir and not force:
-            pickle           =  Path(self.dut_dir)
+            pickle = Path(self.dut_dir)
             if pickle.is_file():
-                existing_dut  =  DutView.load_dut(self.dut_dir)
+                existing_dut = DutView.load_dut(self.dut_dir)
                 self.__dict__.update(existing_dut.__dict__)
 
-        self.sim_dir    = DATA_CONFIG['directories']['simulation']
+        self.sim_dir = DATA_CONFIG["directories"]["simulation"]
 
-        self.technology     = technology
+        self.technology = technology
         self.contact_config = contact_config
-        self.width          = width
-        self.length         = length
-        self.nfinger        = nfinger
-        if (width is not None) and (length is not None) and (not isinstance(length, tuple) and (not isinstance(width, tuple))):
-            self.perimeter  = (self.width+self.length)*2
-            self.area       =  self.width*self.length
+        self.width = width
+        self.length = length
+        self.nfinger = nfinger
+        if (
+            (width is not None)
+            and (length is not None)
+            and (not isinstance(length, tuple) and (not isinstance(width, tuple)))
+        ):
+            self.perimeter = (self.width + self.length) * 2
+            self.area = self.width * self.length
         else:
-            self.perimeter  = None
-            self.area       = None
+            self.perimeter = None
+            self.area = None
 
         # simulation attributes
-        self.t_max              = t_max
+        self.t_max = t_max
         if simulate_on_server is None:
-            self.simulate_on_server = DATA_CONFIG['backend_remote']
+            self.simulate_on_server = DATA_CONFIG["backend_remote"]
         else:
             self.simulate_on_server = simulate_on_server
+        self.zip_result = True
 
-        #in these attributes, store the name of another dut that has been used for deembedding...for documentation this is nice
-        self.open_deembedded_with = '-'
-        self.short_deembedded_with = '-'
+        # in these attributes, store the name of another dut that has been used for deembedding...for documentation this is nice
+        self.open_deembedded_with = "-"
+        self.short_deembedded_with = "-"
 
     def prepare_simulation(self, sweep):
-        """ Creates a simulation folder, appends the sweep to the structure definition and creates a input file in the simulation folder.
+        """Creates a simulation folder, appends the sweep to the structure definition and creates a input file in the simulation folder.
 
         Parameters
         -----------
         sweep : Sweep
         """
-        logging.info("Preparing to simulate the Dut %s the sweep with the hash %s", self.get_hash(), sweep.get_hash())
+        logging.info(
+            "Preparing to simulate the Dut %s the sweep with the hash %s",
+            self.get_hash(),
+            sweep.get_hash(),
+        )
 
+        # set the simulation folder
+        sim_folder = self.get_sim_folder(sweep)
 
         inp_content = self.make_input(sweep)
-
-        #set the simulation folder
-        sim_folder = self.get_sim_folder(sweep)
+        pbs = DATA_CONFIG["server"]["use_pbs"] and DATA_CONFIG["backend_remote"]
+        if pbs:
+            pbs_content = self.make_pbs(sweep)  # needs to be implemented by the DUT
 
         try:
             sim_folder.mkdir(parents=True)
         except OSError:
             logging.debug("Folder already existed! Will be deleted")
-            shutil.rmtree(str(sim_folder))
-            sim_folder.mkdir(parents=True, exist_ok=True)
+            try:
+                shutil.rmtree(str(sim_folder))
+                sim_folder.mkdir(parents=True, exist_ok=True)
+            except FileNotFoundError:
+                raise OSError(
+                    "Simulation folder exists but can not be deleted. Is the location mounted correctly? Hint: Shut down Windows correctly."
+                )
 
-        (sim_folder/self.inp_name).write_text(inp_content)
+        (sim_folder / self.inp_name).write_text(inp_content)
+        if pbs:
+            (sim_folder / "pbs_job").write_text(pbs_content)
 
         # copy va files?
         sim_folder = self.get_sim_folder(sweep)
         if self._copy_va_files:
             for (va_file, va_file_content) in self._list_va_file_contents:
-                (sim_folder/va_file).write_text(va_file_content)
+                (sim_folder / va_file).write_text(va_file_content)
 
         for data_copy in self.list_copy:
             try:
@@ -258,52 +305,56 @@ class DutView(object):
                     shutil.copyfile(data_copy, os.path.join(sim_folder, filename))
                 else:
                     # filename ??? Setting default file name... seems crazy here
-                    (sim_folder / 'datafile.tbl').write_text(data_copy)
+                    (sim_folder / "datafile.tbl").write_text(data_copy)
             except TypeError:
                 file_content = self._write_data_table(data_copy)
-                file_name = sim_folder / 'datafile.tbl'
-                with file_name.open('a') as file_table:
+                file_name = sim_folder / "datafile.tbl"
+                with file_name.open("a") as file_table:
                     file_table.write(file_content)
 
-        print("Prepared a simulation in the folder: \n" + str(sim_folder))
-        logging.info("Prepared a simulation in the folder: \n%s", sim_folder)
+        str_log = " Prepared a simulation in the folder: \n{0:s}".format(str(sim_folder))
+        print(str_log)
+        logging.info(str_log)
 
     def get_start_sim_command(self):
-        """ Returns the command to start the simulation
+        """Returns the command to start the simulation
 
         Returns
         --------
         str
         """
-        return self.sim_command + ' ' + ' '.join(self.sim_args) + ' ' + self.inp_name
+        return self.sim_command + " " + " ".join(self.sim_args) + " " + self.inp_name
 
     def _write_data_table(self, data):
-        """ Option to write a data table for a simulation. As the file type may depend on the simulator this is not implemented here!
-        """
+        """Option to write a data table for a simulation. As the file type may depend on the simulator this is not implemented here!"""
         raise NotImplementedError("Has to be overwritten in the subclass.")
 
+    def make_pbs(self, sweep):
+        """Create a PBS script for the PBS job system. (See: https://albertsk.files.wordpress.com/2011/12/pbs.pdf)"""
+        raise NotImplementedError("Not Implemented for this Dut Class!")
+
     def make_input(self, sweep):
-        """ Joins simulation header with a given Sweep object and returns it.
+        """Joins simulation header with a given Sweep object and returns it.
 
         Parameters
         ----------
         sweep : :class:`DMT.core.sweep.Sweep`
             Sweep specification according to the Sweep class.
         """
-        raise NotImplementedError('DutView does not implement the concrete make_input method!')
+        raise NotImplementedError("DutView does not implement the concrete make_input method!")
 
     def get_hash(self):
-        """ Return a unique hash for this Dut without (!) considering the Sweep.
+        """Return a unique hash for this Dut without (!) considering the Sweep.
 
         Returns
         ----------
         hash : float64
             Hash that corresponds to this DUT.
         """
-        raise NotImplementedError('DutView does not implement the concrete get_hash method!')
+        raise NotImplementedError("DutView does not implement the concrete get_hash method!")
 
     def import_output_data(self, sweep, delete_sim_results=False):
-        """ Read the output files that have been produced while simulating sweep and attach them to self.db.
+        """Read the output files that have been produced while simulating sweep and attach them to self.db.
 
         Parameters
         ----------
@@ -319,11 +370,12 @@ class DutView(object):
         IOError
             If the given sweep can not be read.
         """
-        raise NotImplementedError('DutView does not implement the concrete import_output_data method!')
-
+        raise NotImplementedError(
+            "DutView does not implement the concrete import_output_data method!"
+        )
 
     def validate_simulation_successful(self, sweep):
-        """ Checks if the simulation of the given sweep was successful.
+        """Checks if the simulation of the given sweep was successful.
 
         Parameters
         ----------
@@ -339,14 +391,16 @@ class DutView(object):
         FileNotFoundError
             If the sim log file does not exist.
         """
-        raise NotImplementedError('DutView does not implement the concrete import_output_data method!')
+        raise NotImplementedError(
+            "DutView does not implement the concrete import_output_data method!"
+        )
 
     @property
     def save_dir(self):
         if self.get_hash():
-            return  self.database_dir / (self.name + '_hash_' + str(self.get_hash()))
+            return self.database_dir / (self.name + "_hash_" + str(self.get_hash()))
         else:
-            return  self.database_dir / self.name
+            return self.database_dir / self.name
 
     @property
     def database_dir(self):
@@ -354,37 +408,35 @@ class DutView(object):
 
     @database_dir.setter
     def database_dir(self, new_dir):
-        """ Make sure data is loaded before changing save location of this DutView.
-        """
+        """Make sure data is loaded before changing save location of this DutView."""
         if not self._data:
             self.load_db()
 
         self._database_dir = Path(new_dir)
 
-    def get_db_dir(self, name='db'):
-        """ Returns the name for a db, either use 'db' for regular behavior or use 'sweep.get_hash()' for a db per sweep.
-        """
-        return self.save_dir / (name + '.h5')
+    def get_db_dir(self, name="db"):
+        """Returns the name for a db, either use 'db' for regular behavior or use 'sweep.get_hash()' for a db per sweep."""
+        return self.save_dir / (name + ".h5")
 
     @property
     def dut_dir(self):
-        return self.save_dir / 'dut.p'
+        return self.save_dir / "dut.p"
 
     @property
     def data(self):
-        """ data is a property to ensure loading before usage.
+        """data is a property to ensure loading before usage.
 
         As _data is a dict, the getter is also called before setting. So the separate setter is not necessary.
         If someone tries to set dut.data an attribute error occurs, but setting dict entries is possible directly.
 
         """
-        if not self._data: # empty dict evaluates to false
+        if not self._data:  # empty dict evaluates to false
             self.load_db()
 
         return self._data
 
     def get_sim_folder(self, sweep):
-        """ Returns the simulation folder of the given sweep
+        """Returns the simulation folder of the given sweep
 
         Parameters
         ----------
@@ -396,10 +448,14 @@ class DutView(object):
         str
             Path to the simulationfolder
         """
-        return self.sim_dir / (self.name + str( self.get_hash() )) / (sweep.name + '_' + sweep.get_hash())
+        return (
+            self.sim_dir
+            / (self.name + str(self.get_hash()))
+            / (sweep.name + "_" + sweep.get_hash())
+        )
 
     def delete_sim_results(self, sweep, ignore_errors=False):
-        """ Deletes the simulation results of the given sweep.
+        """Deletes the simulation results of the given sweep.
 
         Parameters
         ----------
@@ -411,43 +467,45 @@ class DutView(object):
         sim_folder = self.get_sim_folder(sweep)
         shutil.rmtree(sim_folder, ignore_errors=ignore_errors)
 
-        logging.info("The simulation data of the sweep %s with the hash %s was deleted", sweep.name, sweep.get_hash())
+        logging.info(
+            "The simulation data of the sweep %s with the hash %s was deleted",
+            sweep.name,
+            sweep.get_hash(),
+        )
 
     def save(self):
-        """ Save this DutView as a pickled .p file and the also the data into the database on the hard drive.
-        """
+        """Save this DutView as a pickled .p file and the also the data into the database on the hard drive."""
         self.save_dir.mkdir(parents=True, exist_ok=True)
-        self._database_dir = self._database_dir.resolve() #convert to absolute path
+        self._database_dir = self._database_dir.resolve()  # convert to absolute path
 
         self.save_db()
-        with open(self.dut_dir, 'wb') as handle:
+        with open(self.dut_dir, "wb") as handle:
             cpickle.dump(self, handle)
         # with warnings.catch_warnings():
-            # warnings.simplefilter('ignore', tb.NaturalNameWarning)
+        # warnings.simplefilter('ignore', tb.NaturalNameWarning)
 
     def __getstate__(self):
-        """ Return state values to be pickled. Implemented according `to <https://www.ibm.com/developerworks/library/l-pypers/index.html>`_ .
+        """Return state values to be pickled. Implemented according `to <https://www.ibm.com/developerworks/library/l-pypers/index.html>`_ .
         Notes
         -----
         ..todo:
             iterate through all properties and throw away the HDFStore objects.
         """
-        d  =  copy.copy(self.__dict__)
+        d = copy.copy(self.__dict__)
 
-        if '_data' in d:
-            del d['_data']
+        if "_data" in d:
+            del d["_data"]
 
         return d
 
     def __setstate__(self, state):
-        """ Return state values to be pickled. Implemented according `to <https://www.ibm.com/developerworks/library/l-pypers/index.html>`_ .
-        """
-        self.__dict__          =  state # pylint: disable=attribute-defined-outside-init
-        self.__dict__['_data'] =  {}
+        """Return state values to be pickled. Implemented according `to <https://www.ibm.com/developerworks/library/l-pypers/index.html>`_ ."""
+        self.__dict__ = state  # pylint: disable=attribute-defined-outside-init
+        self.__dict__["_data"] = {}
 
     @staticmethod
     def load_dut(file_dut):
-        """ Static class method. Loads a DutView object from a pickle file with full path save_dir.
+        """Static class method. Loads a DutView object from a pickle file with full path save_dir.
 
         Parameters
         ----------
@@ -462,7 +520,7 @@ class DutView(object):
         if not isinstance(file_dut, Path):
             file_dut = Path(file_dut)
 
-        with file_dut.open(mode='rb') as handle:
+        with file_dut.open(mode="rb") as handle:
             dut = cpickle.load(handle)
 
         # check the dirs:
@@ -470,16 +528,16 @@ class DutView(object):
         database_dir = file_dut.parent.parent
 
         # check if equal, if not -> changed machine -> broken absolute path
-        if database_dir != dut._database_dir: # pylint: disable=protected-access
-            dut._database_dir = database_dir #pylint: disable=protected-access
+        if database_dir != dut._database_dir:  # pylint: disable=protected-access
+            dut._database_dir = database_dir  # pylint: disable=protected-access
 
         if not os.path.exists(dut.sim_dir):
-            dut.sim_dir = DATA_CONFIG['directories']['simulation']
+            dut.sim_dir = DATA_CONFIG["directories"]["simulation"]
 
         return dut
 
     def add_data(self, data, key=None, force=True, **kwargs):
-        """ Add a measurement or simulation data to the DutView's data.
+        """Add a measurement or simulation data to the DutView's data.
 
         Parameters
         ----------
@@ -500,27 +558,38 @@ class DutView(object):
 
         if key in self.data.keys():
             if force:
-                logging.info('DMT -> DutView -> add_data(): Removed a dataframe with key %s since it was already existent in dut.data.', key)
+                logging.info(
+                    "DMT -> DutView -> add_data(): Removed a dataframe with key %s since it was already existent in dut.data.",
+                    key,
+                )
                 self.remove_data(key)
             else:
-                logging.info('DMT -> DutView -> add_data(): Skipped a dataframe with key %s since it was already existent in dut.data.', key)
+                logging.info(
+                    "DMT -> DutView -> add_data(): Skipped a dataframe with key %s since it was already existent in dut.data.",
+                    key,
+                )
                 return
 
         try:
             self.data[key] = read_data(data, **kwargs)
-        except (OSError,IOError, TypeError) as _error:
+        except (OSError, IOError, TypeError) as _error:
             # could not read file using the general read data!
-            if isinstance(data, DataFrame):
+            if isinstance(data, DataFrame) or isinstance(data, pd.DataFrame):
                 # it is a regular dataframe
+                # prevents pandas bug with non-unique columns:
+                if not data.columns.is_unique:
+                    data = data.loc[:, ~data.columns.duplicated()]
                 self.data[key] = data
             else:
+                # simulation valid?
+                self.validate_simulation_successful(data)
                 # try special import
                 self.import_output_data(data)
 
-        logging.info('DMT -> DutView -> add_data(): Added a dataframe with key %s to the dut.', key)
+        logging.info("DMT -> DutView -> add_data(): Added a dataframe with key %s to the dut.", key)
 
     def remove_data(self, key):
-        """ Remove a measurement or simulation dataframe from the DutView's data.
+        """Remove a measurement or simulation dataframe from the DutView's data.
 
         Parameters
         ----------
@@ -529,8 +598,8 @@ class DutView(object):
         """
         del self.data[key]
 
-    def get_data(self, key='iv', sweep=None):
-        """ Return data stored in the DutView's data.
+    def get_data(self, key="iv", sweep=None):
+        """Return data stored in the DutView's data.
 
         | One needs to specify either:
         | - key      : The data stored under the path key in the dut's database is returned.
@@ -548,13 +617,13 @@ class DutView(object):
             if sweep is None:
                 raise IOError("If you do not know the sweep, just use dut.data[key]..")
 
-            key  =  self.join_key(self.get_sweep_key(sweep), key)
+            key = self.join_key(self.get_sweep_key(sweep), key)
             if key not in self._data.keys():
                 self.load_db(sweep)
 
         else:
             if sweep is not None:
-                key  =  self.join_key(self.get_sweep_key(sweep), key)
+                key = self.join_key(self.get_sweep_key(sweep), key)
 
             if not self._data:
                 self.load_db()
@@ -563,38 +632,60 @@ class DutView(object):
             return self._data[key]
         except KeyError:
             ## missed the starting '/' ?
-            return self._data[self.join_key('', key)]
+            return self._data[self.join_key("", key)]
 
     def del_db(self):
-        """ Delete the DutView's complete database.
-        """
-        for _root, _dirs, files in os.walk(self.save_dir):
+        """Delete the DutView's complete database."""
+        # iterate over the folder in case of self.separate_databases
+        for root, _dirs, files in os.walk(self.save_dir):
             for my_file in files:
-                if my_file.endswith('.h5'):
-                    self.manager.del_db(my_file)
-        logging.info('DMT -> DutView -> del_data(): Deleted a complete database.')
+                if my_file.endswith(".h5"):
+                    self.manager.del_db(root + "/" + my_file)
+        logging.info("DMT -> DutView -> del_data(): Deleted a complete database.")
 
     def del_dut(self):
-        """ Delete the DutView's pickled file.
-        """
+        """Delete the DutView's pickled file."""
         if os.path.exists(self.dut_dir):
             os.remove(self.dut_dir)
 
-        logging.info('DMT -> DutView -> del_dut(): Deleted a pickled DutView object.')
+        logging.info("DMT -> DutView -> del_dut(): Deleted a pickled DutView object.")
+
+    def clean_data(self, fallback=None, **kwargs):
+        """Clean the dataframe columns of the DataFrame objects in this DutMeas objects database.
+
+        Parameters
+        ----------
+        fallback : dict, optional
+            Is used to update self.nodes_fallback for this special key to clean.
 
-    def clean_data(self):
-        """ Iterate through all keys in the DutView's data and try to clean the column names according to the internal DMT format.
         """
+        if fallback is None:
+            fallback = {}
         for key, df in self.data.items():
-            self.data[key] = df.clean_data(self.nodes, self.reference_node)
+            try:
+                df = df.clean_data(
+                    self.nodes,
+                    self.reference_node,
+                    fallback=fallback,
+                    ac_ports=self.ac_ports,
+                    **kwargs,
+                )
+            except AttributeError:
+                df.__class__ = DataFrame  # cast to DMT.DataFrame
+                df = df.clean_data(
+                    self.nodes,
+                    self.reference_node,
+                    fallback=fallback,
+                    ac_ports=self.ac_ports,
+                    **kwargs,
+                )
 
-        logging.info('DMT -> DutView -> clean_all_data(): Cleaned dataframes.')
+            self.data[key] = df
 
     def save_db(self):
-        ''' Write a database for this dut. If it already exists it is overwritten. Does NOT save all keys starting with '_'
-        '''
+        """Write a database for this dut. If it already exists it is overwritten. Does NOT save all keys starting with '_'"""
         if not self._data:
-            return # nothing to do here
+            return  # nothing to do here
 
         if self._separate_databases:
             # find all sweeps in self.data
@@ -608,20 +699,23 @@ class DutView(object):
             for sweep_key in sweep_keys:
                 data_to_save = {}
                 for key, data in self._data.items():
-                    if not key.startswith('_') and self.join_key(*self.split_key(key)[0:-1]) == sweep_key:
+                    if (
+                        not key.startswith("_")
+                        and self.join_key(*self.split_key(key)[0:-1]) == sweep_key
+                    ):
                         data_to_save[key] = data
 
                 self.manager.save_db(self.get_db_dir(sweep_key), data_to_save)
         else:
             data_to_save = {}
             for key, data in self._data.items():
-                if not key.startswith('_'):
+                if not key.startswith("_"):
                     data_to_save[key] = data
 
             self.manager.save_db(self.get_db_dir(), data_to_save)
 
     def load_db(self, sweep=None):
-        ''' Load saved data from the database into the object.
+        """Load saved data from the database into the object.
 
         If the complete dictionary is reloaded, here it is set directly to self._data.
         if separate databases is activated it loads only the given sweep (and it has to be given then!)
@@ -630,7 +724,7 @@ class DutView(object):
         -------------
         sweep : Sweep or str, optional
             In case of separate databases, the sweep must be given either directly or as string name
-        '''
+        """
         if self._separate_databases:
             if sweep is None:
                 return
@@ -651,7 +745,7 @@ class DutView(object):
                 pass
 
     def check_existence_sweep(self, sweep):
-        ''' Return true, if the combination dut+sweep has already been simulated.
+        """Return true, if the combination dut+sweep has already been simulated.
 
         If self.data is None, the database is loaded first.
 
@@ -665,19 +759,19 @@ class DutView(object):
         existence  :  bool
             Is True if the combination dut+sweep has already been simulated.
 
-        '''
-        key_sweep  =  self.get_sweep_key(sweep)
+        """
+        key_sweep = self.get_sweep_key(sweep)
         if self._separate_databases:
-            if not any(key_sweep in data_key for data_key in self.data.keys()): # needs full check
+            if not any(key_sweep in data_key for data_key in self.data.keys()):  # needs full check
                 self.load_db(sweep)
         else:
-            if not self._data: # easy check
+            if not self._data:  # easy check
                 self.load_db()
 
         return any(key_sweep in data_key for data_key in self.data.keys())
 
     def join_key(self, *parts_key):
-        """ Joins the parts of the key into one key for self.dict
+        """Joins the parts of the key into one key for self.dict
 
         Parameters
         -----------
@@ -688,10 +782,10 @@ class DutView(object):
         str
             parts_key[0]/parts_key[1]/.../parts_key[-1]
         """
-        return '/'.join(parts_key)
+        return "/".join(parts_key)
 
     def split_key(self, key):
-        """ Splits the key up in its parts.
+        """Splits the key up in its parts.
 
         Parameters
         -----------
@@ -703,10 +797,10 @@ class DutView(object):
             key splitted at '/'
 
         """
-        return key.split('/')
+        return key.split("/")
 
     def get_sweep_key(self, sweep):
-        """ Key for the dict in dut.data.
+        """Key for the dict in dut.data.
 
         Parameters
         ------------
@@ -718,13 +812,13 @@ class DutView(object):
         key : str
         """
         try:
-            return self.join_key(sweep.get_temperature(), sweep.name + '_' + sweep.get_hash())
+            return self.join_key(sweep.get_temperature(), sweep.name + "_" + sweep.get_hash())
         except AttributeError:
             # given sweep parameter does not have a get_hash(), is it a string?!?
             return sweep
 
     def get_key_temperature(self, key):
-        """ Function that returns the temperature of a given data key. Overwrite this if the measurements differ from the default DMT naming.
+        """Function that returns the temperature of a given data key. Overwrite this if the measurements differ from the default DMT naming.
 
         Default naming is:
 
@@ -750,7 +844,7 @@ class DutView(object):
             if re_temp:
                 try:
                     # always replace "p" with ".", if it is already with ".", it doesnt matter
-                    return round(float(re_temp.group(1).replace('p', '.')), 3)
+                    return round(float(re_temp.group(1).replace("p", ".")), 3)
                 except ValueError:
                     # if a value error in the except clause happens, try the next key part.
                     pass
@@ -760,29 +854,28 @@ class DutView(object):
             re_temp = re.search(r"T[(](.+)[)]K", key_part)
             if re_temp:
                 try:
-                    str_temps = re_temp.group(1).split(',')
+                    str_temps = re_temp.group(1).split(",")
                     return [round(float(str_temp), 3) for str_temp in str_temps]
                 except ValueError:
                     # if a value error in the except clause happens, try the next key part.
                     pass
 
-         # check for range
+        # check for range
         for key_part in key_parts:
             re_temp = re.search(r"T\[(.+)\)K", key_part)
             if re_temp:
                 try:
-                    str_temps = re_temp.group(1).split('-')
+                    str_temps = re_temp.group(1).split("-")
                     return (round(float(str_temp), 3) for str_temp in str_temps)
                 except ValueError:
                     # if a value error in the except clause happens, try the next key part.
                     pass
 
-
         # alternative:
         for key_part in key_parts:
-            if key_part.startswith('T'):
+            if key_part.startswith("T"):
                 try:
-                    return round(float(key_part[1:].replace('p', '.')), 3)
+                    return round(float(key_part[1:].replace("p", ".")), 3)
                 except ValueError:
                     # if a value error in the except clause happens, try the next key part.
                     pass
@@ -790,9 +883,13 @@ class DutView(object):
         # finally as a last escape: direct conversion :(
         for key_part in key_parts:
             try:
-                return round(float(key_part.replace('p', '.')), 3)
+                return round(float(key_part.replace("p", ".")), 3)
             except ValueError:
                 # if a value error in the except clause happens, try the next key part.
                 pass
 
-        raise NameError("DMT -> DutMeas -> get_key_temperature: Was not able to extract the temperature from the key: " + key + ".")
\ No newline at end of file
+        raise NameError(
+            "DMT -> DutMeas -> get_key_temperature: Was not able to extract the temperature from the key: "
+            + key
+            + "."
+        )
