@@ -4,7 +4,7 @@ These are routines and classes which manage the interfaces to other Python packa
 """
 
 ## build tex
-from .latex import build_tex, build_svg, clean_tex_files
+from .latex import build_tex, build_svg, clean_tex_files, build_png
 
 ## pylatex classes
 from .pylatex import Tex
@@ -13,4 +13,4 @@ from .pylatex import CommandBase
 from .pylatex import CommandInput
 from .pylatex import CommandLabel
 from .pylatex import CommandRef
-from .pylatex import CommandRefRange
\ No newline at end of file
+from .pylatex import CommandRefRange
